'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var umiCore = require('@metaplex-foundation/umi-core');
var metaplexAuth = require('@nftstorage/metaplex-auth');
var memory = require('ipfs-car/blockstore/memory');
var nft_storage = require('nft.storage');
var utils = require('./utils.cjs');

/* eslint-disable no-await-in-loop */
class NftStorageUploader {
  constructor(context, options = {}) {
    this.context = context;
    this.payer = options.payer;
    this.token = options.token;
    this.endpoint = options.endpoint;
    this.gatewayHost = options.gatewayHost;
    this.batchSize = options.batchSize ?? 50;
    this.useGatewayUrls = options.useGatewayUrls ?? true;
  }
  async getUploadPrice() {
    return umiCore.lamports(0);
  }
  async upload(files, options = {}) {
    if (this.batchSize <= 0) {
      throw new Error('batchSize must be greater than 0');
    }
    const client = await this.client();
    const blockstore = new memory.MemoryBlockStore();
    const uris = [];
    const numBatches = Math.ceil(files.length / this.batchSize);
    const batches = new Array(numBatches).fill([]).map((_, i) => files.slice(i * this.batchSize, (i + 1) * this.batchSize));
    for (let i = 0; i < batches.length; i += 1) {
      const batch = batches[i];
      const batchLinks = [];
      for (let j = 0; j < batch.length; j += 1) {
        const file = batch[j];
        const blob = new nft_storage.Blob([file.buffer]);
        const node = await nft_storage.NFTStorage.encodeBlob(blob, {
          blockstore
        });
        const fileUri = this.useGatewayUrls ? utils.toGatewayUri(node.cid.toString(), undefined, this.gatewayHost) : utils.toIpfsUri(node.cid.toString());
        uris.push(fileUri);
        batchLinks.push(await utils.toDagPbLink(node, file.uniqueName));
      }
      const batchBlock = await utils.toDirectoryBlock(batchLinks);
      const {
        cid,
        car
      } = await utils.toEncodedCar(batchBlock, blockstore);
      const storeOptions = {};
      if (options.onProgress) {
        const {
          onProgress
        } = options;
        const totalSize = umiCore.getBytesFromGenericFiles(...files);
        let uploadedSize = 0;
        storeOptions.onStoredChunk = size => {
          uploadedSize += size;
          const percent = uploadedSize / totalSize * 100;
          onProgress(Math.min(percent, 100), size);
        };
      }
      const promise = isNFTStorageMetaplexor(client) ? client.storeCar(cid, car, storeOptions) : client.storeCar(car, storeOptions);
      await promise;
    }
    return uris;
  }
  async uploadJson(json) {
    const file = umiCore.createGenericFileFromJson(json);
    const uris = await this.upload([file]);
    return uris[0];
  }
  async client() {
    if (this.token) {
      return new nft_storage.NFTStorage({
        token: this.token,
        endpoint: this.endpoint
      });
    }
    const signer = this.payer ?? this.context.payer;
    const authOptions = {
      mintingAgent: '@metaplex-foundation/umi-plugin-nft-storage',
      solanaCluster: this.context.rpc.getCluster(),
      endpoint: this.endpoint
    };
    return umiCore.isKeypairSigner(signer) ? metaplexAuth.NFTStorageMetaplexor.withSecretKey(signer.secretKey, authOptions) : metaplexAuth.NFTStorageMetaplexor.withSigner(signer.signMessage.bind(signer), signer.publicKey.bytes, authOptions);
  }
}
const isNFTStorageMetaplexor = client => 'storeNFTFromFilesystem' in client;

exports.NftStorageUploader = NftStorageUploader;
//# sourceMappingURL=NftStorageUploader.cjs.map
